@@ -15,7 +15,6 @@
 
 import json
 import os
-import sys
 import ssl
 import socket
 import copy
@@ -202,7 +201,7 @@ class JsonResponse(Response):
 
         try:
             body = json.loads(self.body)
-        except:
+        except Exception:
             raise MalformedResponseError(
                 'Failed to parse JSON',
                 body=self.body,
@@ -227,12 +226,11 @@ class XmlResponse(Response):
             except ValueError:
                 # lxml wants a bytes and tests are basically hard-coded to str
                 body = ET.XML(self.body.encode('utf-8'))
-        except:
+        except Exception:
             raise MalformedResponseError('Failed to parse XML',
                                          body=self.body,
                                          driver=self.connection.driver)
         return body
-
     parse_error = parse_body
 
 
@@ -348,16 +346,21 @@ class Connection(object):
 
     def set_http_proxy(self, proxy_url):
         """
-        Set a HTTP proxy which will be used with this connection.
+        Set a HTTP / HTTPS proxy which will be used with this connection.
 
         :param proxy_url: Proxy URL (e.g. http://<hostname>:<port> without
                           authentication and
-                          http://<username>:<password>@<hostname>:<port> for
-                          basic auth authentication information.
+                          <scheme>://<username>:<password>@<hostname>:<port>
+                          for basic auth authentication information.
         :type proxy_url: ``str``
         """
         self.proxy_url = proxy_url
 
+        # NOTE: Because of the way connection instantion works, we need to call
+        # self.connection.set_http_proxy() here. Just setting "self.proxy_url"
+        # won't work.
+        self.connection.set_http_proxy(proxy_url=proxy_url)
+
     def set_context(self, context):
         if not isinstance(context, dict):
             raise TypeError('context needs to be a dictionary')
@@ -601,8 +604,7 @@ class Connection(object):
                 else:
                     self.connection.request(method=method, url=url, body=data,
                                             headers=headers, stream=stream)
-        except socket.gaierror:
-            e = sys.exc_info()[1]
+        except socket.gaierror as e:
             message = str(e)
             errno = getattr(e, 'errno', None)
 
@@ -619,8 +621,7 @@ class Connection(object):
                 raise socket.gaierror(msg)
             self.reset_context()
             raise e
-        except ssl.SSLError:
-            e = sys.exc_info()[1]
+        except ssl.SSLError as e:
             self.reset_context()
             raise ssl.SSLError(str(e))
 
