@@ -19,18 +19,19 @@ Provides base classes for working with drivers
 
 from __future__ import with_statement
 
-import sys
 import time
 import hashlib
 import os
 import socket
+import random
 import binascii
 
 from libcloud.utils.py3 import b
 
 import libcloud.compute.ssh
 from libcloud.pricing import get_size_price
-from libcloud.compute.types import NodeState, DeploymentError
+from libcloud.compute.types import NodeState, StorageVolumeState,\
+    DeploymentError
 from libcloud.compute.ssh import SSHClient
 from libcloud.common.base import ConnectionKey
 from libcloud.common.base import BaseDriver
@@ -62,12 +63,14 @@ __all__ = [
     'NodeState',
     'NodeSize',
     'NodeImage',
+    'NodeImageMember',
     'NodeLocation',
     'NodeAuthSSHKey',
     'NodeAuthPassword',
     'NodeDriver',
 
     'StorageVolume',
+    'StorageVolumeState',
     'VolumeSnapshot',
 
     # Deprecated, moved to libcloud.utils.networking
@@ -164,7 +167,7 @@ class Node(UuidMixin):
     """
 
     def __init__(self, id, name, state, public_ips, private_ips,
-                 driver, size=None, image=None, extra=None):
+                 driver, size=None, image=None, extra=None, created_at=None):
         """
         :param id: Node ID.
         :type id: ``str``
@@ -175,6 +178,7 @@ class Node(UuidMixin):
         :param state: Node state.
         :type state: :class:`libcloud.compute.types.NodeState`
 
+
         :param public_ips: Public IP addresses associated with this node.
         :type public_ips: ``list``
 
@@ -188,7 +192,10 @@ class Node(UuidMixin):
         :type size: :class:`.NodeSize`
 
         :param image: Image of this node. (optional)
-        :type size: :class:`.NodeImage`
+        :type image: :class:`.NodeImage`
+
+        :param created_at: The datetime this node was created (optional)
+        :type created_at: :class: `datetime.datetime`
 
         :param extra: Optional provider specific attributes associated with
                       this node.
@@ -202,6 +209,7 @@ class Node(UuidMixin):
         self.private_ips = private_ips if private_ips else []
         self.driver = driver
         self.size = size
+        self.created_at = created_at
         self.image = image
         self.extra = extra or {}
         UuidMixin.__init__(self)
@@ -376,6 +384,57 @@ class NodeImage(UuidMixin):
                 % (self.id, self.name, self.driver.name))
 
 
+class NodeImageMember(UuidMixin):
+    """
+    A member of an image. At some cloud providers there is a mechanism
+    to share images. Once an image is shared with another account that
+    user will be a 'member' of the image.
+
+    For example, see the image members schema in the OpenStack Image
+    Service API v2 documentation. https://developer.openstack.org/
+    api-ref/image/v2/index.html#image-members-schema
+
+    NodeImageMember objects are typically returned by the driver for the
+    cloud provider in response to the list_image_members method
+    """
+
+    def __init__(self, id, image_id, state, driver, created=None, extra=None):
+        """
+        :param id: Image member ID.
+        :type id: ``str``
+
+        :param id: The associated image ID.
+        :type id: ``str``
+
+        :param state: State of the NodeImageMember. If not
+                      provided, will default to UNKNOWN.
+        :type state: :class:`.NodeImageMemberState`
+
+        :param driver: Driver this image belongs to.
+        :type driver: :class:`.NodeDriver`
+
+        :param      created: A datetime object that represents when the
+                             image member was created
+        :type       created: ``datetime.datetime``
+
+        :param extra: Optional provided specific attributes associated with
+                      this image.
+        :type extra: ``dict``
+        """
+        self.id = str(id)
+        self.image_id = str(image_id)
+        self.state = state
+        self.driver = driver
+        self.created = created
+        self.extra = extra or {}
+        UuidMixin.__init__(self)
+
+    def __repr__(self):
+        return (('<NodeImageMember: id=%s, image_id=%s, '
+                 'state=%s, driver=%s  ...>')
+                % (self.id, self.image_id, self.state, self.driver.name))
+
+
 class NodeLocation(object):
     """
     A physical location where nodes can be.
@@ -387,7 +446,7 @@ class NodeLocation(object):
     'US'
     """
 
-    def __init__(self, id, name, country, driver):
+    def __init__(self, id, name, country, driver, extra=None):
         """
         :param id: Location ID.
         :type id: ``str``
@@ -400,11 +459,16 @@ class NodeLocation(object):
 
         :param driver: Driver this location belongs to.
         :type driver: :class:`.NodeDriver`
+
+        :param extra: Optional provided specific attributes associated with
+                      this location.
+        :type extra: ``dict``
         """
         self.id = str(id)
         self.name = name
         self.country = country
         self.driver = driver
+        self.extra = extra or {}
 
     def __repr__(self):
         return (('<NodeLocation: id=%s, name=%s, country=%s, driver=%s>')
@@ -460,7 +524,8 @@ class StorageVolume(UuidMixin):
     A base StorageVolume class to derive from.
     """
 
-    def __init__(self, id, name, size, driver, extra=None):
+    def __init__(self, id, name, size, driver,
+                 state=None, extra=None):
         """
         :param id: Storage volume ID.
         :type id: ``str``
@@ -474,6 +539,10 @@ class StorageVolume(UuidMixin):
         :param driver: Driver this image belongs to.
         :type driver: :class:`.NodeDriver`
 
+        :param state: Optional state of the StorageVolume. If not
+                      provided, will default to UNKNOWN.
+        :type state: :class:`.StorageVolumeState`
+
         :param extra: Optional provider specific attributes.
         :type extra: ``dict``
         """
@@ -482,6 +551,7 @@ class StorageVolume(UuidMixin):
         self.size = size
         self.driver = driver
         self.extra = extra
+        self.state = state
         UuidMixin.__init__(self)
 
     def list_snapshots(self):
@@ -545,23 +615,42 @@ class VolumeSnapshot(object):
     """
     A base VolumeSnapshot class to derive from.
     """
-    def __init__(self, id, driver, size=None, extra=None):
+    def __init__(self, id, driver, size=None, extra=None, created=None,
+                 state=None, name=None):
         """
         VolumeSnapshot constructor.
 
         :param      id: Snapshot ID.
         :type       id: ``str``
 
+        :param      driver: The driver that represents a connection to the
+                            provider
+        :type       driver: `NodeDriver`
+
         :param      size: A snapshot size in GB.
         :type       size: ``int``
 
         :param      extra: Provider depends parameters for snapshot.
         :type       extra: ``dict``
+
+        :param      created: A datetime object that represents when the
+                             snapshot was created
+        :type       created: ``datetime.datetime``
+
+        :param      state: A string representing the state the snapshot is
+                           in. See `libcloud.compute.types.StorageVolumeState`.
+        :type       state: ``str``
+
+        :param      name: A string representing the name of the snapshot
+        :type       name: ``str``
         """
         self.id = id
         self.driver = driver
         self.size = size
         self.extra = extra or {}
+        self.created = created
+        self.state = state
+        self.name = name
 
     def destroy(self):
         """
@@ -572,8 +661,8 @@ class VolumeSnapshot(object):
         return self.driver.destroy_volume_snapshot(snapshot=self)
 
     def __repr__(self):
-        return ('<VolumeSnapshot id=%s size=%s driver=%s>' %
-                (self.id, self.size, self.driver.name))
+        return ('<VolumeSnapshot "%s" id=%s size=%s driver=%s state=%s>' %
+                (self.name, self.id, self.size, self.driver.name, self.state))
 
 
 class KeyPair(object):
@@ -628,6 +717,8 @@ class NodeDriver(BaseDriver):
     name = None
     type = None
     port = None
+    website = None
+    api_name = None
     features = {'create_node': []}
 
     """
@@ -644,12 +735,6 @@ class NodeDriver(BaseDriver):
 
     NODE_STATE_MAP = {}
 
-    def __init__(self, key, secret=None, secure=True, host=None, port=None,
-                 api_version=None, **kwargs):
-        super(NodeDriver, self).__init__(key=key, secret=secret, secure=secure,
-                                         host=host, port=port,
-                                         api_version=api_version, **kwargs)
-
     def list_nodes(self):
         """
         List all nodes.
@@ -893,8 +978,7 @@ class NodeDriver(BaseDriver):
                 wait_period=3,
                 timeout=kwargs.get('timeout', NODE_ONLINE_WAIT_TIMEOUT),
                 ssh_interface=ssh_interface)[0]
-        except Exception:
-            e = sys.exc_info()[1]
+        except Exception as e:
             raise DeploymentError(node=node, original_exception=e, driver=self)
 
         ssh_username = kwargs.get('ssh_username', 'root')
@@ -914,11 +998,10 @@ class NodeDriver(BaseDriver):
                     ssh_username=username, ssh_password=password,
                     ssh_key_file=ssh_key_file, ssh_timeout=ssh_timeout,
                     timeout=timeout, max_tries=max_tries)
-            except Exception:
+            except Exception as e:
                 # Try alternate username
                 # Todo: Need to fix paramiko so we can catch a more specific
                 # exception
-                e = sys.exc_info()[1]
                 deploy_error = e
             else:
                 # Script successfully executed, don't try alternate username
@@ -997,9 +1080,9 @@ class NodeDriver(BaseDriver):
                                (optional)
         :type location: :class:`.NodeLocation`
 
-        :param snapshot:  Name of snapshot from which to create the new
-                               volume.  (optional)
-        :type snapshot:  ``str``
+        :param snapshot:  Snapshot from which to create the new
+                          volume.  (optional)
+        :type snapshot: :class:`.VolumeSnapshot`
 
         :return: The newly created volume.
         :rtype: :class:`StorageVolume`
@@ -1007,10 +1090,16 @@ class NodeDriver(BaseDriver):
         raise NotImplementedError(
             'create_volume not implemented for this driver')
 
-    def create_volume_snapshot(self, volume, name):
+    def create_volume_snapshot(self, volume, name=None):
         """
         Creates a snapshot of the storage volume.
 
+        :param volume: The StorageVolume to create a VolumeSnapshot from
+        :type volume: :class:`.VolumeSnapshot`
+
+        :param name: Name of created snapshot (optional)
+        :type name: `str`
+
         :rtype: :class:`VolumeSnapshot`
         """
         raise NotImplementedError(
@@ -1062,6 +1151,9 @@ class NodeDriver(BaseDriver):
         """
         Destroys a snapshot.
 
+        :param snapshot: The snapshot to delete
+        :type snapshot: :class:`VolumeSnapshot`
+
         :rtype: :class:`bool`
         """
         raise NotImplementedError(
@@ -1218,7 +1310,7 @@ class NodeDriver(BaseDriver):
         key_file_path = os.path.expanduser(key_file_path)
 
         with open(key_file_path, 'r') as fp:
-            key_material = fp.read()
+            key_material = fp.read().strip()
 
         return self.import_key_pair_from_string(name=name,
                                                 key_material=key_material)
@@ -1233,8 +1325,9 @@ class NodeDriver(BaseDriver):
         raise NotImplementedError(
             'delete_key_pair not implemented for this driver')
 
-    def wait_until_running(self, nodes, wait_period=3, timeout=600,
-                           ssh_interface='public_ips', force_ipv4=True):
+    def wait_until_running(self, nodes, wait_period=3,
+                           timeout=600, ssh_interface='public_ips',
+                           force_ipv4=True, ex_list_nodes_kwargs=None):
         """
         Block until the provided nodes are considered running.
 
@@ -1260,10 +1353,17 @@ class NodeDriver(BaseDriver):
         :param force_ipv4: Ignore IPv6 addresses (default is True).
         :type force_ipv4: ``bool``
 
+        :param ex_list_nodes_kwargs: Optional driver-specific keyword arguments
+                                     which are passed to the ``list_nodes``
+                                     method.
+        :type ex_list_nodes_kwargs: ``dict``
+
         :return: ``[(Node, ip_addresses)]`` list of tuple of Node instance and
                  list of ip_address on success.
         :rtype: ``list`` of ``tuple``
         """
+        ex_list_nodes_kwargs = ex_list_nodes_kwargs or {}
+
         def is_supported(address):
             """
             Return True for supported address.
@@ -1289,7 +1389,7 @@ class NodeDriver(BaseDriver):
         uuids = set([node.uuid for node in nodes])
 
         while time.time() < end:
-            all_nodes = self.list_nodes()
+            all_nodes = self.list_nodes(**ex_list_nodes_kwargs)
             matching_nodes = list([node for node in all_nodes
                                    if node.uuid in uuids])
 
@@ -1302,8 +1402,11 @@ class NodeDriver(BaseDriver):
 
             running_nodes = [node for node in matching_nodes
                              if node.state == NodeState.RUNNING]
-            addresses = [filter_addresses(getattr(node, ssh_interface))
-                         for node in running_nodes]
+            addresses = []
+            for node in running_nodes:
+                node_addresses = filter_addresses(getattr(node, ssh_interface))
+                if len(node_addresses) >= 1:
+                    addresses.append(node_addresses)
 
             if len(running_nodes) == len(uuids) == len(addresses):
                 return list(zip(running_nodes, addresses))
@@ -1343,7 +1446,18 @@ class NodeDriver(BaseDriver):
         if 'password' in self.features['create_node']:
             value = os.urandom(16)
             value = binascii.hexlify(value).decode('ascii')
-            return NodeAuthPassword(value, generated=True)
+
+            # Some providers require password to also include uppercase
+            # characters so convert some characters to uppercase
+            password = ''
+            for char in value:
+                if not char.isdigit() and char.islower():
+                    if random.randint(0, 1) == 1:
+                        char = char.upper()
+
+                password += char
+
+            return NodeAuthPassword(password, generated=True)
 
         if auth:
             raise LibcloudError(
@@ -1383,14 +1497,23 @@ class NodeDriver(BaseDriver):
         while time.time() < end:
             try:
                 ssh_client.connect()
-            except SSH_TIMEOUT_EXCEPTION_CLASSES:
-                e = sys.exc_info()[1]
+            except SSH_TIMEOUT_EXCEPTION_CLASSES as e:
+                # Errors which represent fatal invalid key files which should
+                # be propagated to the user
                 message = str(e).lower()
-                expected_msg = 'no such file or directory'
+                invalid_key_msgs = [
+                    'no such file or directory',
+                    'invalid key',
+                    'not a valid ',
+                ]
 
-                if isinstance(e, IOError) and expected_msg in message:
-                    # Propagate (key) file doesn't exist errors
-                    raise e
+                # Propagate (key) file doesn't exist errors
+                # NOTE: Paramiko only supports PEM private key format
+                # See https://github.com/paramiko/paramiko/issues/1313
+                # for details
+                for invalid_key_msg in invalid_key_msgs:
+                    if invalid_key_msg in message:
+                        raise e
 
                 # Retry if a connection is refused, timeout occurred,
                 # or the connection fails due to failed authentication.
@@ -1455,11 +1578,10 @@ class NodeDriver(BaseDriver):
         while tries < max_tries:
             try:
                 node = task.run(node, ssh_client)
-            except Exception:
+            except Exception as e:
                 tries += 1
 
                 if tries >= max_tries:
-                    e = sys.exc_info()[1]
                     raise LibcloudError(value='Failed after %d tries: %s'
                                         % (max_tries, str(e)), driver=self)
             else:
