@@ -95,6 +95,7 @@ class DigitalOcean_v2_NodeDriver(DigitalOcean_v2_BaseDriver,
     EX_CREATE_ATTRIBUTES = ['backups',
                             'ipv6',
                             'private_networking',
+                            'tags',
                             'ssh_keys']
 
     def list_images(self):
@@ -111,9 +112,22 @@ class DigitalOcean_v2_NodeDriver(DigitalOcean_v2_BaseDriver,
         data = self._paginated_request('/v2/account/keys', 'ssh_keys')
         return list(map(self._to_key_pair, data))
 
-    def list_locations(self):
+    def list_locations(self, ex_available=True):
+        """
+        List locations
+
+        :param ex_available: Only return locations which are available.
+        :type ex_evailable: ``bool``
+        """
+        locations = []
         data = self._paginated_request('/v2/regions', 'regions')
-        return list(map(self._to_location, data))
+        for location in data:
+            if ex_available:
+                if location.get('available'):
+                    locations.append(self._to_location(location))
+            else:
+                locations.append(self._to_location(location))
+        return locations
 
     def list_nodes(self):
         data = self._paginated_request('/v2/droplets', 'droplets')
@@ -138,6 +152,7 @@ class DigitalOcean_v2_NodeDriver(DigitalOcean_v2_BaseDriver,
         * `backups`: ``bool`` defaults to False
         * `ipv6`: ``bool`` defaults to False
         * `private_networking`: ``bool`` defaults to False
+        * `tags`: ``list`` of ``str`` tags
         * `user_data`: ``str`` for cloud-config data
         * `ssh_keys`: ``list`` of ``int`` key ids or ``str`` fingerprints
 
@@ -493,8 +508,126 @@ class DigitalOcean_v2_NodeDriver(DigitalOcean_v2_BaseDriver,
                                       method='DELETE')
         return res.status == httplib.NO_CONTENT
 
+    def ex_get_node_details(self, node_id):
+        """
+        Lists details of the specified server.
+
+        :param       node_id: ID of the node which should be used
+        :type        node_id: ``str``
+
+        :rtype: :class:`Node`
+        """
+        data = self._paginated_request(
+            '/v2/droplets/{}'.format(node_id), 'droplet'
+        )
+        return self._to_node(data)
+
+    def ex_create_floating_ip(self, location):
+        """
+        Create new floating IP reserved to a region.
+
+        The newly created floating IP will not be associated to a Droplet.
+
+        See https://developers.digitalocean.com/documentation/v2/#floating-ips
+
+        :param location: Which data center to create the floating IP in.
+        :type location: :class:`.NodeLocation`
+
+        :rtype: :class:`DigitalOcean_v2_FloatingIpAddress`
+        """
+        attr = {'region': location.id}
+        resp = self.connection.request('/v2/floating_ips',
+                                       data=json.dumps(attr), method='POST')
+        return self._to_floating_ip(resp.object['floating_ip'])
+
+    def ex_delete_floating_ip(self, ip):
+        """
+        Delete specified floating IP
+
+        :param      ip: floating IP to remove
+        :type       ip: :class:`DigitalOcean_v2_FloatingIpAddress`
+
+        :rtype: ``bool``
+        """
+        resp = self.connection.request('/v2/floating_ips/{}'.format(ip.id),
+                                       method='DELETE')
+        return resp.status == httplib.NO_CONTENT
+
+    def ex_list_floating_ips(self):
+        """
+        List floating IPs
+
+        :rtype: ``list`` of :class:`DigitalOcean_v2_FloatingIpAddress`
+        """
+        return self._to_floating_ips(
+            self._paginated_request('/v2/floating_ips', 'floating_ips')
+        )
+
+    def ex_get_floating_ip(self, ip):
+        """
+        Get specified floating IP
+
+        :param      ip: floating IP to get
+        :type       ip: ``str``
+
+        :rtype: :class:`DigitalOcean_v2_FloatingIpAddress`
+        """
+        floating_ips = self.ex_list_floating_ips()
+        matching_ips = [x for x in floating_ips if x.ip_address == ip]
+        if not matching_ips:
+            raise ValueError('Floating ip %s not found' % ip)
+        return matching_ips[0]
+
+    def ex_attach_floating_ip_to_node(self, node, ip):
+        """
+        Attach the floating IP to the node
+
+        :param      node: node
+        :type       node: :class:`Node`
+
+        :param      ip: floating IP to attach
+        :type       ip: ``str`` or :class:`DigitalOcean_v2_FloatingIpAddress`
+
+        :rtype: ``bool``
+        """
+        data = {
+            'type': 'assign',
+            'droplet_id': node.id
+        }
+        resp = self.connection.request(
+            '/v2/floating_ips/%s/actions' % ip.ip_address,
+            data=json.dumps(data), method='POST'
+        )
+        return resp.status == httplib.CREATED
+
+    def ex_detach_floating_ip_from_node(self, node, ip):
+        """
+        Detach a floating IP from the given node
+
+        Note: the 'node' object is not used in this method but it is added
+        to the signature of ex_detach_floating_ip_from_node anyway so it
+        conforms to the interface of the method of the same name for other
+        drivers like for example OpenStack.
+
+        :param      node: Node from which the IP should be detached
+        :type       node: :class:`Node`
+
+        :param      ip: Floating IP to detach
+        :type       ip: :class:`DigitalOcean_v2_FloatingIpAddress`
+
+        :rtype: ``bool``
+        """
+        data = {
+            'type': 'unassign'
+        }
+        resp = self.connection.request(
+            '/v2/floating_ips/%s/actions' % ip.ip_address,
+            data=json.dumps(data), method='POST'
+        )
+        return resp.status == httplib.CREATED
+
     def _to_node(self, data):
-        extra_keys = ['memory', 'vcpus', 'disk', 'region', 'image',
+        extra_keys = ['memory', 'vcpus', 'disk', 'image', 'size',
                       'size_slug', 'locked', 'created_at', 'networks',
                       'kernel', 'backup_ids', 'snapshot_ids', 'features',
                       'tags']
@@ -518,7 +651,7 @@ class DigitalOcean_v2_NodeDriver(DigitalOcean_v2_BaseDriver,
         for key in extra_keys:
             if key in data:
                 extra[key] = data[key]
-
+        extra['region'] = data.get('region', {}).get('name')
         node = Node(id=data['id'], name=data['name'], state=state,
                     public_ips=public_ips, private_ips=private_ips,
                     created_at=created, driver=self, extra=extra)
@@ -545,8 +678,9 @@ class DigitalOcean_v2_NodeDriver(DigitalOcean_v2_BaseDriver,
                              extra=extra)
 
     def _to_location(self, data):
+        extra = data.get('features', [])
         return NodeLocation(id=data['slug'], name=data['name'], country=None,
-                            driver=self)
+                            extra=extra, driver=self)
 
     def _to_size(self, data):
         extra = {'vcpus': data['vcpus'],
@@ -573,3 +707,44 @@ class DigitalOcean_v2_NodeDriver(DigitalOcean_v2_BaseDriver,
         return VolumeSnapshot(id=data['id'], name=data['name'],
                               size=data['size_gigabytes'],
                               driver=self, extra=extra)
+
+    def _to_floating_ips(self, obj):
+        return [self._to_floating_ip(ip) for ip in obj]
+
+    def _to_floating_ip(self, obj):
+        return DigitalOcean_v2_FloatingIpAddress(
+            # There is no ID, but the IP is unique so we can use that
+            id=obj['ip'],
+            ip_address=obj['ip'],
+            node_id=obj['droplet']['id'] if obj['droplet'] else None,
+            extra={
+                'region': obj['region'],
+            },
+            driver=self
+        )
+
+
+class DigitalOcean_v2_FloatingIpAddress(object):
+    """
+    Floating IP info.
+    """
+
+    def __init__(self, id, ip_address, node_id=None, extra=None, driver=None):
+        self.id = str(id)
+        self.ip_address = ip_address
+        self.extra = extra
+        self.node_id = node_id
+        self.driver = driver
+
+    def delete(self):
+        """
+        Delete this floating IP
+
+        :rtype: ``bool``
+        """
+        return self.driver.ex_delete_floating_ip(self)
+
+    def __repr__(self):
+        return ('<DigitalOcean_v2_FloatingIpAddress: id=%s, ip_addr=%s,'
+                ' driver=%s>'
+                % (self.id, self.ip_address, self.driver))
